library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use STD.TEXTIO.ALL;
use IEEE.NUMERIC_STD.ALL;


entity TB_VGAtest is
 Port (we	 : buffer std_logic:= '1'; 
		 hsync : buffer std_logic;
	    vsync : buffer std_logic;
		 reset   : buffer std_logic:= '0'; 
		 data_in : buffer std_logic_vector(7 downto 0) := (others => '0');--%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		 display_width : buffer integer := 100;									--NASTAVI ŽELENO ŠIRINO ZASLONA  - kot št. znakov						
		 display_height: buffer integer := 75;										--NASTAVI ŽELENO DOLŽINO ZASLONA - kot št. znakov
		 en      : buffer std_logic;													--%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		 clr_screen:out std_logic;
		 ascii   : buffer std_logic_vector(6 downto 0):= (others => '0');
		 adr_wr	:buffer  unsigned(13 downto 0) := (others => '0');
		 adr_read: buffer unsigned (13 downto 0) := (others => '0'));
end TB_VGAtest;


architecture Behavioral of TB_VGAtest is

component VGA is 
Generic (display_width : integer;								
		   display_height: integer);
Port (clk   : in std_logic;
	   hsync : out std_logic;
	   vsync : out std_logic;    
	   cx		: out unsigned(9 downto 0):= (others => '0');
	   cy		: out unsigned(9 downto 0):= (others => '0');
	   en    : out std_logic := '0');		  
	end component;	


component VGAtext is
 Generic (display_width : integer;								
		    display_height: integer);
 Port (clk      : in std_logic;
		 rgb3     : out unsigned(2 downto 0) := "010";
		 data_in  : in std_logic_vector(7 downto 0) := (others => '0');
		 we		 : in std_logic;		 
		 reset    : in std_logic;
		 cx, cy   : in unsigned (9 downto 0) := (others => '0');
		 en       : in std_logic := '0';
		 clr_screen:out std_logic;
		 pix      : buffer std_logic;
		 ascii    : buffer std_logic_vector(6 downto 0);	  	 
		 adr_wr	 : buffer unsigned (13 downto 0) := (others => '0');
		 adr_read : buffer unsigned (13 downto 0) := (others => '0'));		
end component;
 
 
component asciiRAM is
 Port (clk     : in std_logic;
		 we		: in std_logic;
		 data_in : in std_logic_vector(7 downto 0) := (others => '0');
		 reset   : in std_logic := '0';
		 ascii	: out std_logic_vector (6 downto 0);
		 adr_read: in unsigned (13 downto 0) := (others => '0');		 
		 adr_wr : in unsigned(13 downto 0) := (others => '0');
		 clr_screen:in std_logic);
end component;

	signal clk : std_logic;
	signal rgb :unsigned(2 downto 0);
	signal pix : std_logic := '0';
	constant clk_period : time := 10 ns;

	signal i : unsigned(14 downto 0) := (others => '0');  --velikost steveca !

	signal cx : unsigned(9 downto 0);
	signal cy :  unsigned(9 downto 0);

	signal st_znakov: integer:=0;

------------------Branje besedila iz tekstovne datoteke---------------------
																													
	constant ram_depth : natural := (display_width * display_height);
	constant ram_width : natural := 8;																	
	 
	type ram_type is array (0 to ram_depth - 1)
	of std_logic_vector(ram_width - 1 downto 0);
  
function init_ram_hex return ram_type is
    file text_file: text open read_mode is "besedilo.txt";
    variable text_line: line;
    variable ram_content: ram_type:= ((others=> (others=> '0')));
    variable c: character;
    variable index: integer:= 0;
    variable hex_val: std_logic_vector(7 downto 0);
    variable vGoodRead: boolean:= true;								--%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    variable display_height: integer:= 75;							--Prepiši sem parameter (display_height) iz VGA
begin																				--%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    L1:for i in 0 to (display_height - 1) loop
        if(not endfile(text_file)) then
            readline(text_file, text_line);    
            L2:while (vGoodRead = true) loop
                if (index < ram_depth) then
					 
                    read(text_line, c, vGoodRead);
						  
                    case c is
								when ' ' => hex_val := X"20";when '!' => hex_val := X"21";when '"' => hex_val := X"22";when '#' => hex_val := X"23";when '$' => hex_val := X"24";when '%' => hex_val := X"25";when '&' => hex_val := X"26";when ''' => hex_val := X"27";when '(' => hex_val := X"28";when ')' => hex_val := X"29";when '*' => hex_val := X"2A";when '+' => hex_val := X"2B";when ',' => hex_val := X"2C";when '-' => hex_val := X"2D";when '.' => hex_val := X"2E";when '/' => hex_val := X"2F";when '0' => hex_val := X"30";when '1' => hex_val := X"31";when '2' => hex_val := X"32";when '3' => hex_val := X"33";when '4' => hex_val := X"34";when '5' => hex_val := X"35";when '6' => hex_val := X"36";when '7' => hex_val := X"37";when '8' => hex_val := X"38";when '9' => hex_val := X"39";when ':' => hex_val := X"3A";when ';' => hex_val := X"3B";when '<' => hex_val := X"3C";when '=' => hex_val := X"3D";when '>' => hex_val := X"3E";when '?' => hex_val := X"3F";when '@' => hex_val := X"40";when 'A' => hex_val := X"41";when 'B' => hex_val := X"42";when 'C' => hex_val := X"43";when 'D' => hex_val := X"44";when 'E' => hex_val := X"45";when 'F' => hex_val := X"46";when 'G' => hex_val := X"47";when 'H' => hex_val := X"48";when 'I' => hex_val := X"49";when 'J' => hex_val := X"4A";when 'K' => hex_val := X"4B";when 'L' => hex_val := X"4C";when 'M' => hex_val := X"4D";when 'N' => hex_val := X"4E";when 'O' => hex_val := X"4F";when 'P' => hex_val := X"50";when 'Q' => hex_val := X"51";when 'R' => hex_val := X"52";when 'S' => hex_val := X"53";when 'T' => hex_val := X"54";when 'U' => hex_val := X"55";when 'V' => hex_val := X"56";when 'W' => hex_val := X"57";when 'X' => hex_val := X"58";when 'Y' => hex_val := X"59";when 'Z' => hex_val := X"5A";when '[' => hex_val := X"5B";when '\' => hex_val := X"5C";when ']' => hex_val := X"5D";when '^' => hex_val := X"5E";when '_' => hex_val := X"5F";when '`' => hex_val := X"60";when 'a' => hex_val := X"61";when 'b' => hex_val := X"62";when 'c' => hex_val := X"63";when 'd' => hex_val := X"64";when 'e' => hex_val := X"65";when 'f' => hex_val := X"66";when 'g' => hex_val := X"67";when 'h' => hex_val := X"68";when 'i' => hex_val := X"69";when 'j' => hex_val := X"6A";when 'k' => hex_val := X"6B";when 'l' => hex_val := X"6C";when 'm' => hex_val := X"6D";when 'n' => hex_val := X"6E";when 'o' => hex_val := X"6F";when 'p' => hex_val := X"70";when 'q' => hex_val := X"71";when 'r' => hex_val := X"72";when 's' => hex_val := X"73";when 't' => hex_val := X"74";when 'u' => hex_val := X"75";when 'v' => hex_val := X"76";when 'w' => hex_val := X"77";when 'x' => hex_val := X"78";when 'y' => hex_val := X"79";when 'z' => hex_val := X"7A";when '{' => hex_val := X"7B";when '|' => hex_val := X"7C";when '}' => hex_val := X"7D";when '~' => hex_val := X"7E";
								when others =>hex_val:= X"60";
						  end case;
                    ram_content(index)(7 downto 0):= hex_val;
                    index:= index + 1;    
                end if;    
             end loop L2;
                index:= index - 1;    
                ram_content(index)(7 downto 0):= X"0A";
                index:= index + 1;                
                vGoodRead:= true;
        else
            exit;
        end if;            
    end loop L1;
    return ram_content;
end function;
  
 signal ram_hex : ram_type := init_ram_hex;
-------------------------------------------------------------------------------------------
 
 
begin
 
u2: VGAtext
generic map (display_width, display_height)
port map (clk, rgb, data_in, we, reset, cx, cy, en, clr_screen, pix, ascii,  adr_wr, adr_read );

u1: VGA  
generic map (display_width, display_height)
port map (clk=>clk, hsync=>hsync, vsync=>vsync, cx=>cx, cy=>cy, en=>en);


clk_p: process
begin
	clk <= '0';
	wait for clk_period/2;
	clk <= '1';
	wait for clk_period/2;	
end process;
   

 
stevec_p: process(clk)
begin
	if rising_edge(clk) then
		if i = (i'range => '1')	then
		else
			i <= (i + 1);					
		end if;
	end if;
end process;


 st_znakov<=ram_depth;
	 

	 
process(clk)
begin
	if rising_edge(clk) then
		if i < st_znakov then 
			data_in <= ram_hex(to_integer(unsigned(i)))(7 downto 0);
			we <= '1';
		else
			we <= '0';
		end if;
		
----------Odsek za testiranje reseta------------
		
--		if i = 20 then			--Nastavi kdaj se proži reset			
--			reset <= '1';
--		else
--			reset <= '0';
--		end if;
------------------------------------------------
	end if;
end process;
	
	
izhod: process(clk)
	file slika: text open write_mode is "slika.txt";
	variable str: line;
	variable c: character;
	variable cnt: unsigned(10 downto 0) := (others => '0');  --velikost steveca!
begin
	if rising_edge(clk) then
		if i >= (ram_depth) then
			if en = '1' then
				c := '+';
				if rgb="000" then
					c := '-';
				elsif rgb="111" then
					c := 'X';
				end if;	  
				write(str, c, right, 1);
				cnt := cnt + 1;
				if cnt = (display_width*8) then
					writeline(slika, str);
					cnt:=(others=>'0');
				end if;
			end if;
		end if;
	end if;
end process;
	
	
end Behavioral;

