library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


entity VGAtext is						
Generic (display_width : integer:=100;
		   display_height: integer:=75);
 Port (clk      : in std_logic;
		 rgb3     : out unsigned(2 downto 0) := "010";
		 data_in  : in std_logic_vector(7 downto 0) := (others => '0');
		 we		 : in std_logic;		 
		 reset    : in std_logic;
		 cx, cy   : in unsigned (9 downto 0) := (others => '0');
		 en       : in std_logic := '0';
		 clr_screen:buffer std_logic :='0';
		 pix      : buffer std_logic;
		 ascii    : buffer std_logic_vector(6 downto 0);	  	 
		 adr_wr	 : buffer unsigned (13 downto 0) := (others => '0');
		 adr_read : buffer unsigned (13 downto 0) := (others => '0'));
end VGAtext;


architecture Behavioral of VGAtext is
 
component asciiRAM is
 Port (clk     : in std_logic;
		 we		: in std_logic;
		 data_in : in std_logic_vector(7 downto 0) := (others => '0');
		 reset   : in std_logic := '0';
		 ascii	: out std_logic_vector (6 downto 0):= (others => '0');
		 adr_read: in unsigned(13 downto 0) := (others => '0');		 
		 adr_wr  : in unsigned(13 downto 0) := (others => '0');
		 clr_screen:in std_logic);
end component;
	
component charROM is
 Port (ascii  : in std_logic_vector (6 downto 0);
       cx, cy : in unsigned (9 downto 0) := (others => '0');
       pix    : out std_logic);
end component;
 
	signal disp_width : integer:=100;						--%% Prepiši display_width %%
																		--%%%%%%%%%%%%%%%%
	signal chars_per_line	:  integer := 80;				--Nastavi velikost
	signal chars_per_column	:  integer := 60;				--		podokna
																		--%%%%%%%%%%%%%%%%
	signal X	:  integer := 0;
	signal Y	:  integer := 0;
	signal X_adr  :  integer := 0;	
	signal Y_adr  :  integer := 0;
	signal X_pos  : integer := 0;	
	signal Y_pos  : integer := 0;
	signal X_width	:  integer := 0;
	signal Y_height:  integer := 0;



	type t_State is (s_ReadWrite, s_Reset, s_ClearScreen); 	
	signal state : t_State;
	signal st : unsigned (13 downto 0) := (others => '0');   --!!! pozor velikost

	signal adr_st : unsigned (2 downto 0) := (others => '0');
	signal window_st : unsigned (2 downto 0) := (others => '0');
	--signal text_colour: unsigned (5 downto 0) := (others => '1'); --naredi 4 bitno pa da se zraven v pomnilnik shranjuje, z ukazom spremenimo...
	--signal background_colour: unsigned (5 downto 0) := (others => '0');
	
	signal clear_screen : std_logic := '0';
	
begin	



u3: charROM port map (ascii, cx, cy, pix);

u4: asciiRAM port map (clk, we, data_in, reset, ascii, adr_read, adr_wr, clr_screen);

clr_screen <= clear_screen;

process(clk)
begin
	if rising_edge(clk) then
	
		case state is
		
			when s_ReadWrite =>
			
			
				if (we = '1') and adr_wr < (display_width * display_height - 1) then
				
				
					------------------Nastavitev koordinat vpiši kot npr.: x30,50y -----------------------
					if data_in = X"58" or data_in = X"78" then		        --(x)						    						
						X_pos <= 0;
						Y_pos <= 0; 
						adr_st <= "001";
					
				
					elsif data_in = X"59" or data_in = X"79" then           --(y)                    
						adr_st <= "000";
						X_adr <= X_pos;
						Y_adr <= Y_pos;
						X <= X_pos;
						Y <= Y_pos;                     
					--------------------------------------------------------------------------------------								
							
					--------------------Nastavitev velikosti okna vpiši kot npr.: +10,20- ----------------							
					elsif data_in = X"2B" then		                          --(+)					   										
						window_st <= "001";
						X_width <= 0;
						Y_height <= 0;


					elsif data_in = X"2D" then		                          --(-)					   	
						window_st <= "000";                         
						chars_per_line <= X_width;
						chars_per_column <= Y_height;                     
					--------------------------------------------------------------------------------------
					
					--------------------------------Izbris izbranega podokna------------------------------
					elsif data_in = X"24" then					--($)					    
						clear_screen <= '1';					
					--------------------------------------------------------------------------------------
						
					--------------------------------Ukaz za novo vrstico----------------------------------
					elsif data_in = X"0A" then								        --(\n)
						if Y_adr = (Y + chars_per_column - 1) then
							X_adr <= X;
							Y_adr <= Y;
						else						   
							X_adr <= X;
							Y_adr <= Y_adr + 1;
						end if;
					--------------------------------------------------------------------------------------
					
					--------------------------------Ukaz BACKSPACE----------------------------------------	
					elsif data_in = X"08" then										--(\b)	
						if  X_adr = X then
							if Y_adr > Y then
								X_adr <= (X + chars_per_line - 1);
								Y_adr <= Y_adr - 1;
							else
								X_adr <= X;
								Y_adr <= Y;
							end if;
						else
							X_adr <= X_adr - 1;
						end if;
					--------------------------------------------------------------------------------------  
					
					-----------------------Ostalo---------------------------------------------------------
					else                                                                                             
						if adr_st > 0 then                  --za nastavitev koordinat
							adr_st <= adr_st + 1;
						end if;
						
						if window_st > 0 then               --za nastavitev velikosti okna
							window_st <= window_st + 1;
						end if; 
	  
						if data_in <= X"7E"  then  			--upoštevanje le prvih 127 ASCII znakov  
--                    if (X_adr = X and data_in = X"20") = false then                     --neizpisovanje presledka na začetku vrstice                                 
								 if X_adr >= (X + chars_per_line - 1) then	--preskok v novo vrstico
									  X_adr <= X;
									  Y_adr <= Y_adr + 1;
								 elsif adr_wr >= ((Y + chars_per_column  - 1) * display_width + X + chars_per_line - 1) then	  --in na začetek okna
--									  X_adr <= X;		--za simulacijo ni dobro
--									  Y_adr <= Y;		--za simulacijo ni dobro
								 else                                   
									  X_adr <= X_adr + 1;
								 end if;
--                   end if;
						end if;						 	   
					end if;   
					--------------------------------------------------------------------------------------
					
							
							
					----------------------------Nastavitev koordinat-----------------------------------------------
					if (adr_st > 0 and adr_st <= 2) and (data_in >= X"30" and data_in <= X"39") then						
						X_pos <= X_pos * 10 + to_integer(unsigned(data_in)) - 48;
					elsif (adr_st > 2 and adr_st <= 5) and (data_in >= X"30" and data_in <= X"39") then
						Y_pos <= Y_pos * 10 + to_integer(unsigned(data_in)) - 48;
					end if;
					-----------------------------------------------------------------------------------------------
									
					----------------------------Nastavitev velikosti okna------------------------------------------
					if (window_st > 0 and window_st <= 2) and (data_in >= X"30" and data_in <= X"39") then						
						X_width <= X_width * 10 + to_integer(unsigned(data_in)) - 48;
					elsif (window_st > 2 and window_st <= 5) and (data_in >= X"30" and data_in <= X"39") then
						Y_height <= Y_height * 10 + to_integer(unsigned(data_in)) - 48;
					end if;
					-----------------------------------------------------------------------------------------------
					
					
--					---------da ne pobegnemo iz vidnega okna - samodejen preskok v novo vrstico--------------------
--					if X_adr >= (chars_per_line-1+X) then
--						X_adr <= X;
--						Y_adr <= Y_adr+1;
--					end if; 
--					-----------------------------------------------------------------------------------------------						
				end if;
						
						
				---Pogoji za prehod med stanji-----------------------------
				if reset = '0' then
					if clear_screen = '1' then
						state <= s_ClearScreen;
						X_adr <= X;
						Y_adr <= Y;
					else
						state <= s_ReadWrite;
					end if;
				else						
					state <= s_Reset;
					X_adr <= 0;
					Y_adr <= 0;
				end if;
				-----------------------------------------------------------
						
						
			
			when s_ClearScreen =>
			
				----------da ne pobegnemo iz vidnega okna - samodejen preskok v novo vrstico----------------------
				if X_adr = (chars_per_line-1+X) then	
					X_adr <= X;
					Y_adr <= Y_adr+1;
				else
					X_adr <= X_adr + 1;
				end if;	
				
				---Pogoji za prehod med stanji----------------------------- 
				if reset = '0' then
					if adr_wr >= ((Y + chars_per_column - 1) * display_width + chars_per_line - 1 + X) then
						clear_screen <= '0';
						state <= s_ReadWrite;
						X_adr <= X;
						Y_adr <= Y;
					else	
						state <= s_ClearScreen;						
					end if;
				else
					state <= s_Reset;
					X_adr <= 0;
					Y_adr <= 0;
				end if;
				-----------------------------------------------------------
				
				
			when s_Reset =>
			
				if X_adr = (display_width-1) then	
					X_adr <= 0;
					Y_adr <= Y_adr+1;
				else
					X_adr <= X_adr + 1;
				end if;	
				
			---Pogoji za prehod med stanji----------------------------- 
				if adr_wr = (display_height * display_width-1) then
					if clear_screen = '1' then
						state <= s_ClearScreen;
						X_adr <= X;
						Y_adr <= Y;
					else
						state <= s_ReadWrite;
						X_adr <= X;
						Y_adr <= Y;
					end if;
				else
					state <= s_Reset;
				end if;
				-----------------------------------------------------------	
				
				
				
		end case;
		
	end if;
		adr_wr <= to_unsigned(X_adr + display_width *Y_adr, adr_wr'length);
		adr_read <= to_unsigned(to_integer(cx(9 downto 3)) + display_width*(to_integer(cy(9 downto 3))), adr_wr'length);
end process;
	

	
process(clk)
begin
	if rising_edge(clk) then
--		if cx >= (X*8) and cx < (8*chars_per_line + X*8) and cy >= (Y*8) and cy < (8*chars_per_column + Y*8) then  --območje podokna
			if en = '1' then																	 --območje zaslona
				if pix = '1' then
					rgb3 <= "111";
				else 	
					rgb3 <= "000";
				end if;
			else rgb3 <= "000"; --<= "010"
			end if;
--		else
--		   rgb3 <= "000";
--		end if;
	end if;
end process;

		
end Behavioral;
