library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


entity asciiRAM is 

Port (clk      : in std_logic;
		 we		: in std_logic;
		 data_in : in std_logic_vector(7 downto 0) := (others => '0');
		 reset   : in std_logic := '0';
		 ascii	: out std_logic_vector(6 downto 0) := (others => '0');
		 adr_read: in unsigned (13 downto 0) := (others => '0');		 
		 adr_wr  : in unsigned(13 downto 0) := (others => '0');
		 clr_screen:in std_logic);
end asciiRAM;		


architecture Behavioral of asciiRAM is

	type memory is array(7499 downto 0) of std_logic_vector(7 downto 0);  --za zaslon 800 x 600
	signal RAM : memory := ((others => (others => '0')));
	
	type t_State is (s_ReadWrite, s_Reset, s_ClearScreen); 	
	signal state : t_State;
	
begin

process(clk)
begin
	if rising_edge(clk) then					
		case state is
		
			when s_ReadWrite =>
			
				if (we = '1') then
					RAM(to_integer(unsigned(adr_wr))) <= data_in;
				end if;	
				
				---Pogoji za prehod med stanji----------------------------- 
					if Reset = '0' then
						if clr_screen = '1' then
							state <= s_ClearScreen;
						else
							state <= s_ReadWrite;
						end if;
					else						
						state <= s_Reset;
					end if;
				-----------------------------------------------------------		
						
			when s_Reset =>
			
				RAM(to_integer(unsigned(adr_wr))) <= X"03";
				
				---Pogoji za prehod med stanji-----------------------------
				if adr_wr >= (100*75-1) then	--(display_width * display_height - 1)	
					if clr_screen = '1' then
						state <= s_ClearScreen;
					else
						state <= s_ReadWrite;
					end if;
				else
					state <= s_Reset;
				end if;
				-----------------------------------------------------------
				
			when s_ClearScreen =>
				
				RAM(to_integer(unsigned(adr_wr))) <= X"01";
				
				---Pogoji za prehod med stanji-----------------------------
				if reset = '0' then
					if clr_screen = '0' then	
						state <= s_ReadWrite;
					else
						state <= s_ClearScreen;
					end if;
				else
					state <= s_Reset;
				end if;
				-----------------------------------------------------------
		end case;
		
		ascii <= RAM(to_integer(unsigned(adr_read)))(6 downto 0);
		
	end if;
end process;

end Behavioral;