----------------------------------------------------------------------------------
-- Company:  FE, 2015-18
-- Engineer: A. Trost
-- 
-- Create Date: 17/11/2015
-- Design Name: Sistem z razsiritveno plosco, VGA
--  clk 50MHz
-- 
-- Revision: 2.1
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity sistem is
	Port( clk : in STD_LOGIC;                  -- 50 MHz ura
			clkout : out STD_LOGIC; 				 -- signali V/I plošče
			addr : out STD_LOGIC_VECTOR(1 DOWNTO 0);
			data : inout STD_LOGIC_VECTOR(7 DOWNTO 0);
			led : out std_logic_vector(7 downto 0);
			cx		: buffer UNSIGNED(9 downto 0):= (others => '0');
			cy		: buffer UNSIGNED(9 downto 0):= (others => '0');
			en    : buffer std_logic := '0'
		 );
end sistem;

architecture opis of sistem is
 signal hsync, vsync: std_logic;        -- sinhroinizacija VGA
 signal x,y: unsigned(11 downto 0);     -- koordinate trenutne tocke
 signal rgbout: unsigned(5 downto 0);   -- barvni izhod VGA 
 
 signal num: unsigned(3 downto 0);      -- stevilo za izpis na LED (4-bit)
 signal key: unsigned(3 downto 0);      -- stanje tipk
 signal rgb: unsigned(2 downto 0);      -- 3-bitne barve za VGA 
 
 signal vhod: std_logic_vector(7 downto 0);
 signal we, key1: std_logic := '0';
 
 -- dodaj deklaracije signalov
 
	component vhod8 is
		port (
			source : out std_logic_vector(7 downto 0)   -- source
		);
	end component vhod8;

  
begin

 U0: entity work.vmesnik port map (
		clk => clk,
		hsync => hsync,
		vsync => vsync,
		rgb => rgbout,
		num => num,
		key => key,
		clkout => clkout,
		addr => addr,
		data => data  );


rgbout <= rgb(2) & rgb(2) & rgb(1) & rgb(1) & rgb(0) & rgb(0);


 U1: entity work.VGAtext generic map (
      display_width => 100,
		display_height => 75) 
	port map (
		clk => clk, 
--		hsync => hsync, 
--		vsync => vsync,
      rgb3 => rgb,	
		data_in => vhod,
		we => we,
		reset => key(0),
		cx =>cx,
		cy =>cy,
		en=>en
		);
		
		
 
 U2: entity work.vga generic map (
      display_width => 100,
		display_height => 75) 
	port map (
		clk => clk, 
		hsync => hsync, 
		vsync => vsync,
		cx =>cx,
		cy =>cy,
		en=>en
		);
   

 u10 : component vhod8
		port map (
			source => vhod  -- sources.source
		); 
 
 
 
process(clk)
begin
	if rising_edge(clk) then
	   key1 <= key(1);
		
		if key1='0' and key(1)='1' then
		  we <= '1';
		else
		  we <= '0';
	   end if;		
		
	end if;
end process;
 
 
----------------------------------------------------------------------------------------------
 num <= key;   -- prikazuj stanje tipk
 led <= vhod;
 
-- dodaj kodo za rgb...

 
end opis;